# Author: Yuehao Bai | University of Michigan | Feb 2021
# -----------------------------------------------------
# simulation with many moment inequalities
# generate graphs with multiple betas
# -----------------------------------------------------
rm(list = ls())
cat("\014")
library(doParallel)
library(foreach)
registerDoParallel(cores = as.numeric(Sys.getenv("SLURM_CPUS_ON_NODE")[1]))
getDoParWorkers()
getDoParName()
source("functions.manymoments.R")

aid <- Sys.getenv('SLURM_ARRAY_TASK_ID')
args <- as.numeric(commandArgs(T))
model <- args[1]
p <- args[2]
rho <- args[3]

set.seed(123)

n <- 100
# p.vec <- c(40, 100, 200)
# rho.vec <- c(0, 0.5, 0.9)
# eps.dist.vec <- c("N", "t", "U")
# model.vec <- 1:1
theta.vec <- c(0, 0.2)
n.iter <- 10000
n.boot <- 10000
outfile <- paste0("outputs/beta-model", model, "-p", p, "-rho", rho, ".txt")
logfile <- paste0("logs/log", aid, ".txt")
close(file(outfile, open = "w"))
close(file(logfile, open = "w"))

################ grid of betas
beta.vec <- seq(0, 0.025, by = 0.001)
n.beta <- length(beta.vec)
cat(
  paste("model", model, "p", p, "rho", rho, "\n"),
  file = outfile,
  append = T
)

if (model %in% c(1, 2)) {
  Sigma <- diag(p) + rho * matrix(1, p, p)
} else {
  Sigma <- diag(p)
  Sigma <- rho ^ abs(row(Sigma) - col(Sigma))
}

A <- chol(Sigma)

rej.vec <-
  foreach (iter = 1:n.iter, .combine = rbind) %dopar% {
    cat(paste("starting iteration", iter, "\n"),
        file = logfile,
        append = T)
    data.obs.0 <-
      genBaseline(model, A, theta = theta.vec[1], p = p)
    rej.mat.0 <-
      testsHDbetas(data.obs.0,
                   beta.vec = beta.vec,
                   n.boot = n.boot)
    data.obs.1 <-
      genBaseline(model, A, theta = theta.vec[2], p = p)
    rej.mat.1 <-
      testsHDbetas(data.obs.1,
                   beta.vec = beta.vec,
                   n.boot = n.boot)
    c(t(cbind(rej.mat.0, rej.mat.1)))
  }

rej.mat <- matrix(colSums(rej.vec), nrow = n.beta, byrow = T)
write.table(
  rej.mat,
  outfile,
  append = T,
  row.names = F,
  col.names = F
)
