# Author: Yuehao Bai | University of Michigan | Feb 2021
# -----------------------------------------------------
# simulation with many moment inequalities
# generate tables 1--4 in the paper
# -----------------------------------------------------
rm(list = ls())
cat("\014")
library(doParallel)
library(foreach)
registerDoParallel(cores = as.numeric(Sys.getenv("SLURM_CPUS_ON_NODE")[1]))
getDoParWorkers()
getDoParName()
source("functions.manymoments.R")

aid <- Sys.getenv('SLURM_ARRAY_TASK_ID')
args <- as.numeric(commandArgs(T))
model <- args[1]
p <- args[2]
rho <- args[3]

set.seed(123)

n <- 100
# p.vec <- c(40, 100, 200)
# rho.vec <- c(0, 0.5, 0.9)
# eps.dist.vec <- c("N", "t", "U")
# model.vec <- 1:1
theta.vec <- c(0, 0.2)
n.iter <- 10000
n.boot <- 1000
logfile <- paste0("logs/log", aid, ".txt")
close(file(logfile, open = "w"))

####################### Tables 1--4
if (model %in% c(1, 2)) {
  Sigma <- diag(p) + rho * matrix(1, p, p)
} else {
  Sigma <- diag(p)
  Sigma <- rho ^ abs(row(Sigma) - col(Sigma))
}

A <- chol(Sigma)

rej.mat <-
  foreach (iter = 1:n.iter, .combine = rbind) %dopar% {
    cat(paste("starting iteration", iter, "\n"),
        file = logfile,
        append = T)
    data.obs.0 <-
      genBaseline(model, A, theta = theta.vec[1], p = p)
    rej.vec.0 <- testsHD(data.obs.0, beta.rsw = 0.001, n.boot = n.boot)
    data.obs.1 <-
      genBaseline(model, A, theta = theta.vec[2], p = p)
    rej.vec.1 <- testsHD(data.obs.1, beta.rsw = 0.001, n.boot = n.boot)
    c(
      rej.vec.0,
      rej.vec.0[1] >= rej.vec.0[3],
      rej.vec.0[2] >= rej.vec.0[4],
      rej.vec.1,
      rej.vec.1[1] >= rej.vec.1[3],
      rej.vec.1[2] >= rej.vec.1[4]
    )
  }

rej.prob <- colSums(rej.mat)
write.table(
  matrix(c(model, p, rho, rej.prob), nrow = 1),
  "outputs/tables-same-beta.txt",
  append = T,
  row.names = F,
  col.names = F
)